@@ -3,7 +3,7 @@
  *		  When the mpars structure is changed,
  *		  change these define's too.
  *
- *		$Id: configsym.h,v 1.5 2006/04/02 09:52:30 al-guest Exp $
+ *		$Id: configsym.h,v 1.6 2007-10-10 20:18:20 al-guest Exp $
  *
  *		This file is part of the minicom communications package,
  *		Copyright 1991-1995 Miquel van Smoorenburg.
@@ -13,6 +13,10 @@
  *		as published by the Free Software Foundation; either version
  *		2 of the License, or (at your option) any later version.
  *
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ *
  *	When adding parameter macros here, remember to also update
  *	their default values in the pars structure mpars in rwconf.c.
  *
