@@ -22,6 +22,7 @@
  * kubota@debian.or.jp 07/98  - Added option -C to start capturing from the
  *				command line
  * jl  09.07.98 added option -S to start a script at startup
+ * mark.einon@gmail.com 16/02/11 - Added option to timestamp terminal output
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -58,6 +59,8 @@ static void signore(int sig)
 }
 #endif /*DEBUG*/
 
+static int line_timestamp;
+
 /*
  * Sub - menu's.
  */
@@ -840,32 +843,59 @@ static void helpthem(void)
     "Report bugs to <minicom-devel@lists.alioth.debian.org>.\n"), CONFDIR);
 }
 
+static void set_addlf(int val)
+{
+  vt_set(val, -1, -1, -1, -1, -1, -1, -1);
+}
+
 /* Toggle linefeed addition.  Can be called through the menu, or by a macro. */
 void toggle_addlf(void)
 {
   addlf = !addlf;
-  vt_set(addlf, -1, -1, -1, -1, -1, -1);
+  set_addlf(addlf);
 }
 
-static void set_addlf(int val)
+static void set_local_echo(int val)
 {
-  vt_set(val, -1, -1, -1, -1, -1, -1);
+  vt_set(-1, -1, -1, -1, val, -1 ,-1, -1);
 }
 
 /* Toggle local echo.  Can be called through the menu, or by a macro. */
 void toggle_local_echo(void)
 {
   local_echo = !local_echo;
-  vt_set(-1, -1, -1, -1, local_echo, -1 ,-1);
+  set_local_echo(local_echo);
 }
 
-static void set_local_echo(int val)
+static void set_line_timestamp(int val)
 {
-  vt_set(-1, -1, -1, -1, val, -1 ,-1);
+  vt_set(-1, -1, -1, -1 ,-1, -1, -1, val);
+}
+
+/* Toggle host timestamping on/off */
+static void toggle_line_timestamp(void)
+{
+  ++line_timestamp;
+  line_timestamp %= TIMESTAMP_LINE_NR_OF_OPTIONS;
+  set_line_timestamp(line_timestamp);
 }
 
 /* -------------------------------------------- */
 
+static void do_iconv_just_copy(char **inbuf, size_t *inbytesleft,
+                               char **outbuf, size_t *outbytesleft)
+{
+  while (*outbytesleft && *inbytesleft)
+    {
+      *(*outbuf) = *(*inbuf);
+      ++(*outbuf);
+      ++(*inbuf);
+      --(*outbytesleft);
+      --(*inbytesleft);
+    }
+}
+
+#ifdef HAVE_ICONV
 static iconv_t iconv_rem2local;
 static int iconv_enabled;
 
@@ -899,8 +929,11 @@ static void init_iconv(const char *remote_charset)
            "%s//TRANSLIT", tmp);
   local_charset[sizeof(local_charset) - 1] = 0;
 
-  //printf("Remote charset: %s\n", remote_charset);
-  //printf("Local charset: %s\n", local_charset);
+  if (0)
+    {
+      printf("Remote charset: %s\n", remote_charset);
+      printf("Local charset: %s\n", local_charset);
+    }
 
   iconv_rem2local = iconv_open(local_charset, remote_charset);
   if (iconv_rem2local != (iconv_t)-1)
@@ -916,16 +949,15 @@ void do_iconv(char **inbuf, size_t *inbytesleft,
   if (!iconv_enabled
       || iconv(iconv_rem2local,
                inbuf, inbytesleft,
-               outbuf, outbytesleft) == (size_t)-1) {
-    memcpy(*outbuf, *inbuf,
-           *outbytesleft < *inbytesleft ? *outbytesleft : *inbytesleft);
+               outbuf, outbytesleft) == (size_t)-1)
+    {
+      do_iconv_just_copy(inbuf, inbytesleft, outbuf, outbytesleft);
 
-    // in case of error re-init conversion descriptor
-    // (will this work?)
-    if (iconv_enabled)
-      iconv(iconv_rem2local, NULL, NULL, NULL, NULL);
-    return;
-  }
+      // in case of error re-init conversion descriptor
+      // (will this work?)
+      if (iconv_enabled)
+	iconv(iconv_rem2local, NULL, NULL, NULL, NULL);
+    }
 }
 
 static void close_iconv(void)
@@ -933,6 +965,28 @@ static void close_iconv(void)
   if (iconv_enabled)
     iconv_close(iconv_rem2local);
 }
+#else
+
+int using_iconv(void)
+{
+  return 0;
+}
+
+static void init_iconv(const char *remote_charset)
+{
+  (void)remote_charset;
+}
+
+void do_iconv(char **inbuf, size_t *inbytesleft,
+              char **outbuf, size_t *outbytesleft)
+{
+  do_iconv_just_copy(inbuf, inbytesleft, outbuf, outbytesleft);
+}
+
+static void close_iconv(void)
+{
+}
+#endif
 /* -------------------------------------------- */
 
 
@@ -999,6 +1053,7 @@ int main(int argc, char **argv)
   stdattr = XA_NORMAL;
   us = NULL;
   addlf = 0;
+  line_timestamp = 0;
   wrapln = 0;
   disable_online_time = 0;
   local_echo = 0;
@@ -1180,7 +1235,7 @@ int main(int argc, char **argv)
             exit(1);
           }
           docap = 1;
-          vt_set(addlf, -1, docap, -1, -1, -1, -1);
+          vt_set(addlf, -1, docap, -1, -1, -1, -1, -1);
           break;
         case 'S': /* start Script */
           strncpy(scr_name, optarg, 33);
@@ -1222,7 +1277,7 @@ int main(int argc, char **argv)
 
   if (screen_iso && screen_ibmpc)
     /* init VT */
-    vt_set(-1, -1, -1, -1, -1, -1, 1);
+    vt_set(-1, -1, -1, -1, -1, -1, 1, -1);
 
   /* Avoid fraude ! */	
   for (s = use_port; *s; s++)
@@ -1387,6 +1442,7 @@ int main(int argc, char **argv)
 
   set_local_echo(local_echo);
   set_addlf(addlf);
+  set_line_timestamp(line_timestamp);
 
   /* The main loop calls do_terminal and gets a function key back. */
   while (!quit) {
@@ -1486,7 +1542,7 @@ dirty_goto:
           if (c == 1)
             docap = 0;
         }
-        vt_set(addlf, -1, docap, -1, -1, -1, -1);
+        vt_set(addlf, -1, docap, -1, -1, -1, -1, -1);
         break;
       case 'p': /* Set parameters */
         get_bbp(P_BAUDRATE, P_BITS, P_PARITY, P_STOPB, 0);
@@ -1510,11 +1566,31 @@ dirty_goto:
           init_emul(c, 1);
         break;
       case 'w': /* Line wrap on-off */
-        c = (!us->wrap);
-        vt_set(addlf, c, docap, -1, -1, -1, -1);
+        c = !us->wrap;
+        vt_set(addlf, c, docap, -1, -1, -1, -1, -1);
         s = c ? _("Linewrap ON") : _("Linewrap OFF");
 	status_set_display(s, 0);
         break;
+      case 'n': /* Line timestamp */
+	toggle_line_timestamp();
+        switch (line_timestamp)
+          {
+          default:
+          case TIMESTAMP_LINE_OFF:
+            s = _("Timestamp OFF");
+            break;
+          case TIMESTAMP_LINE_SIMPLE:
+            s = _("Timestamp every line (simple)");
+            break;
+          case TIMESTAMP_LINE_EXTENDED:
+            s = _("Timestamp every line (extended)");
+            break;
+          case TIMESTAMP_LINE_PER_SECOND:
+            s = _("Timestamp lines every second");
+            break;
+          }
+        status_set_display(s, 0);
+        break;
       case 'o': /* Configure Minicom */
         (void) config(0);
         break;
@@ -1533,8 +1609,7 @@ dirty_goto:
       case 'i': /* Re-init, re-open portfd. */
         cursormode = (cursormode == NORMAL) ? APPL : NORMAL;
         keyboard(cursormode == NORMAL ? KCURST : KCURAPP, 0);
-        if (st)
-          curs_status();
+        curs_status();
         break;
       case 'y': /* Paste file */
 	paste_file();
@@ -1569,8 +1644,7 @@ dirty_goto:
   mc_wclose(st, 0);
   mc_wclose(stdwin, 1);
   keyboard(KUNINSTALL, 0);
-  if (lockfile[0])
-    unlink(lockfile);
+  lockfile_remove();
   close(portfd);
 
   if (quit != NORESET && P_CALLIN[0])
